==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/database/arch/p_arch_metadata.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Database submodule - middle layer for arch specific code
 *
 * Notes:
 *  - For now, it is only for x86
 *
 * Timeline:
 *  - Created: 26.VIII.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../p_lkrg_main.h"


void p_dump_CPU_metadata(void *_p_arg) {

   p_ed_pcfi_cpu(0);

#ifdef CONFIG_X86

   p_dump_x86_metadata(_p_arg);

#elif defined(CONFIG_ARM64)

   p_dump_arm64_metadata(_p_arg);

#elif defined(CONFIG_ARM)

   p_dump_arm_metadata(_p_arg);

#endif

}

int p_register_arch_metadata(void) {

   int p_ret = P_LKRG_SUCCESS;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_register_arch_metadata>\n");

   P_SYM(p_core_kernel_text) = (int (*)(unsigned long))P_SYM(p_kallsyms_lookup_name)("core_kernel_text");

   if (!P_SYM(p_core_kernel_text)) {
      p_print_log(P_LKRG_ERR,
             "[ED] ERROR: Can't find 'core_kernel_text' function :( Exiting...\n");
      p_ret = P_LKRG_GENERAL_ERROR;
      goto p_register_arch_metadata_out;
   }

#ifdef P_LKRG_RUNTIME_CODE_INTEGRITY_SWITCH_IDT_H

   if (p_install_switch_idt_hook()) {
      p_print_log(P_LKRG_CRIT,
             "ERROR: Can't hook 'switch_idt' function :( "
             "It's OK, but tracelogs might be not supported - if enabled, it might generate FP! (depends on the kernel version)\n");
      //
      // p_ret = P_LKRG_GENERAL_ERROR;
      // goto error path
      //
      // The reason why we do not stop initialization here (error condition)
      // is because this can only happen in kernel < 3.10 - which is rare and acceptable.
      //
   }

#endif

   spin_lock_init(&p_db.p_jump_label.p_jl_lock);
   /*
    * This is not an arch specific hook, but it's a good place to register it
    */
   if (p_install_arch_jump_label_transform_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook arch_jump_label_transform function :(\n");
      p_ret = P_LKRG_GENERAL_ERROR;
      goto p_register_arch_metadata_out;
   }

#ifdef P_LKRG_CI_ARCH_JUMP_LABEL_TRANSFORM_APPLY_H
   /*
    * This is not an arch specific hook, but it's a good place to register it
    */
   if (p_install_arch_jump_label_transform_apply_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook arch_jump_label_transform_apply function :(\n");
      p_ret = P_LKRG_GENERAL_ERROR;
      goto p_register_arch_metadata_out;
   }
#endif

p_register_arch_metadata_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_register_arch_metadata> (p_ret => %d)\n",p_ret);

   return p_ret;
}


int p_unregister_arch_metadata(void) {

   int p_ret = P_LKRG_SUCCESS;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_register_arch_metadata>\n");

#ifdef P_LKRG_RUNTIME_CODE_INTEGRITY_SWITCH_IDT_H
   p_uninstall_switch_idt_hook();
#endif

   /*
    * This is not an arch specific hook, but it's a good place to deregister it
    */
   p_uninstall_arch_jump_label_transform_hook();
#ifdef P_LKRG_CI_ARCH_JUMP_LABEL_TRANSFORM_APPLY_H
   p_uninstall_arch_jump_label_transform_apply_hook();
#endif

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_register_arch_metadata> (p_ret => %d)\n",p_ret);

   return p_ret;
}

==> lkrg-0.9.0.tar.gz: lkrg-0.9.0/src/modules/database/arch/p_arch_metadata.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Database submodule - middle layer for arch specific code
 *
 * Notes:
 *  - For now, it is only for x86
 *
 * Timeline:
 *  - Created: 26.VIII.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../p_lkrg_main.h"


void p_dump_CPU_metadata(void *_p_arg) {

   p_ed_pcfi_cpu(0);

#ifdef CONFIG_X86

   p_dump_x86_metadata(_p_arg);

#elif defined(CONFIG_ARM64)

   p_dump_arm64_metadata(_p_arg);

#elif defined(CONFIG_ARM)

   p_dump_arm_metadata(_p_arg);

#endif

}

int p_register_arch_metadata(void) {

   P_SYM(p_core_kernel_text) = (int (*)(unsigned long))P_SYM(p_kallsyms_lookup_name)("core_kernel_text");

   if (!P_SYM(p_core_kernel_text)) {
      p_print_log(P_LKRG_ERR,
             "[ED] ERROR: Can't find 'core_kernel_text' function :( Exiting...\n");
      return P_LKRG_GENERAL_ERROR;
   }

#ifdef P_LKRG_RUNTIME_CODE_INTEGRITY_SWITCH_IDT_H

   if (p_install_switch_idt_hook()) {
      p_print_log(P_LKRG_CRIT,
             "ERROR: Can't hook 'switch_idt' function :( "
             "It's OK, but tracelogs might be not supported - if enabled, it might generate FP! (depends on the kernel version)\n");
      //
      // p_ret = P_LKRG_GENERAL_ERROR;
      // goto error path
      //
      // The reason why we do not stop initialization here (error condition)
      // is because this can only happen in kernel < 3.10 - which is rare and acceptable.
      //
   }

#endif

   /*
    * This is not an arch specific hook, but it's a good place to register it
    */
   if (p_install_arch_jump_label_transform_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook 'arch_jump_label_transform' function :(\n");
      return P_LKRG_GENERAL_ERROR;
   }

#ifdef P_LKRG_CI_ARCH_JUMP_LABEL_TRANSFORM_APPLY_H
   /*
    * This is not an arch specific hook, but it's a good place to register it
    */
   if (p_install_arch_jump_label_transform_apply_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook 'arch_jump_label_transform_apply' function :(\n");
      return P_LKRG_GENERAL_ERROR;
   }
#endif

#if defined(CONFIG_DYNAMIC_FTRACE)
   /*
    * Same for FTRACE
    */
   if (p_install_ftrace_modify_all_code_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook 'ftrace_modify_all_code' function :(\n");
      return P_LKRG_GENERAL_ERROR;
   }
#endif

#if defined(CONFIG_FUNCTION_TRACER)
   if (p_install_ftrace_enable_sysctl_hook()) {
      p_print_log(P_LKRG_ERR,
             "ERROR: Can't hook 'ftrace_enable_sysctl' function :(\n");
      return P_LKRG_GENERAL_ERROR;
   }
#endif

   return P_LKRG_SUCCESS;
}


int p_unregister_arch_metadata(void) {

#ifdef P_LKRG_RUNTIME_CODE_INTEGRITY_SWITCH_IDT_H
   p_uninstall_switch_idt_hook();
#endif

   /*
    * This is not an arch specific hook, but it's a good place to deregister it
    */
   p_uninstall_arch_jump_label_transform_hook();
#ifdef P_LKRG_CI_ARCH_JUMP_LABEL_TRANSFORM_APPLY_H
   p_uninstall_arch_jump_label_transform_apply_hook();
#endif
#if defined(CONFIG_DYNAMIC_FTRACE)
   p_uninstall_ftrace_modify_all_code_hook();
#endif
#if defined(CONFIG_FUNCTION_TRACER)
   p_uninstall_ftrace_enable_sysctl_hook();
#endif

   return P_LKRG_SUCCESS;
}
